/* 
 * FILE --- matrix.hh
 *
 * PURPOSE --- Header file for a class for implementing a matrix of doubles
 * together with matrix operations such as multiplication, transpose, inverse,
 * etc. It is intended to be used with the control library.
 */

// Prevent Multiple Inclusion
#ifndef MATRIX_HH
#define MATRIX_HH

// Include Files
#include <stdlib.h>

// Local Definitions and Macros
#define MATRIX_NO_ERROR 0
#define MEMORY_ERROR 1
#define INVALID_DIMENTION 2
#define ILLEGAL_INDEX 3
#define DIVIDE_BY_ZERO 4
#define NONSQUARE_MATRIX 5
#define NONINVERTIBLE_MATRIX 6
#define SIZE_MISMATCH 7
#define INVALID_SET_MATRIX 8


/* Definition of the class */
class MATRIX 
{
public:
  /* Consrtuctor --- Fills the matrix with single value specified by _val*/
  MATRIX(int _row = 2, int _col = 2, double _val = 0.0);

  /* Consrtuctor --- Fills the matrix with the values specified by _elements. 
     The dimension of _elements should be _row-by-_col */
  MATRIX(int _row, int _col, double **_elements);

  /* Copy Constructor */
  MATRIX(const MATRIX &_matrix);

  // Destructor
  ~MATRIX();

  void allocate_memory(void);
  void free_memory(void);

  // Return the # of rows or columns
  inline int get_num_rows() const { return(row); }
  inline int get_num_cols() const { return(col); }

  // Assign all the elements of the matrix
  // _elements should be of appropriate dimension
  void set_matrix(double _val); // to the value given by _val
  void set_matrix(const double **_elements); // to values given by _elements
  // The following is to be used for vectors (1-col or 1-row matrices)
  void set_matrix(const double *_elements); // to values given by _elements

  // Assign the diagonal elements of the matrix
  // _elements should be of appropriate dimension
  void set_diagonal(double _val); // to the value given by _val
  void set_diagonal(const double *_elements); // to values given by _elements

  // Assign the elements of the specified row of the matrix
  // _elements should be of appropriate dimension
  void set_row(int _row, double _val); // to the value given by _val
  void set_row(int _row, const double *_elements); // to values given by _elements

  // Assign the elements of the specified column of the matrix
  // _elements should be of appropriate dimension
  void set_col(int _col, double _val); // to the value given by _val
  void set_col(int _col, const double *_elements); // to values given by _elements

  // Return the value of the element at specified index
  double get_element(int _row, int _col) const; 

  // Assign the element at specified index the value given by _val
  void set_element(int _row, int _col, double _val);

  /* Add, subtract, multiply or devide the element at specified index 
     by the value given by _val */
  void add_to_element(int _row, int _col, double _val);
  void subtract_from_element(int _row, int _col, double _val);
  void multiply_element(int _row, int _col, double _val);
  void divide_element(int _row, int _col, double _val);

  /* Add, subtract, multiply or devide all the elements of the specified
     row by the value given by _val */
  void add_to_row(int _row, double _val);
  void subtract_from_row(int _row, double _val);
  void multiply_row(int _row, double _val);
  void devide_row(int _row, double _val);

  /* Add, subtract, multiply or devide all the elements of the specified
     column by the value given by _val */
  void add_to_col(int _col, double _val);
  void subtract_from_col(int _col, double _val);
  void multiply_col(int _col, double _val);
  void devide_col(int _col, double _val);

  // Add a row (column) multiplied by a scalar to another row (column)
  void add_rows(int _source_row, int _destination_row, double gain);
  void add_cols(int _source_col, int _destination_col, double gain);

  // Overload some of the operators
  MATRIX &operator=(const MATRIX &_matrix2);
  const MATRIX &operator+=(const MATRIX &_matrix2);
  const MATRIX &operator-=(const MATRIX &_matrix2);
  const MATRIX &operator*=(const double _val);

  MATRIX operator+(const MATRIX &_matrix2) const;
  MATRIX operator-(const MATRIX &_matrix2) const;
  MATRIX operator*(const MATRIX &_matrix2) const;

  MATRIX operator*(const double _val);

  friend MATRIX operator*(const double _val, const MATRIX &_matrix);

  int matrix_err;    // Wariable to hold the error

private:
  double** elements;   // Two dimentional array that holds the elements
  int row, col;        // The # of rows and columns
};

MATRIX transpose(const MATRIX &_matrix);    // Take the transpose
MATRIX inv(const MATRIX &_matrix);  // Take the inverse (square matrices)
MATRIX pinv(const MATRIX &_matrix);  // Take the pseudo inverse
double trace(const MATRIX &_matrix);   // Find the trace
double norm(const MATRIX &_matrix);   // Find the Frobenious norm

//MATRIX operator+(const MATRIX &_matrix1, const MATRIX &_matrix2);
//MATRIX operator-(const MATRIX &_matrix1, const MATRIX &_matrix2);
//MATRIX operator*(const MATRIX &_matrix1, const MATRIX &_matrix2);

MATRIX operator*(const double _val, const MATRIX &_matrix);
//MATRIX operator*(const MATRIX &_matrix, const double _val);

typedef MATRIX matrix;

#endif // MATRIX_HH

